# Sid Meier's Civilization 4
# Copyright Firaxis Games 2005
# Orion's Army Mod
from CvPythonExtensions import *
import BugUtil
import BugCore
import CvUtil
import CvTechChooser
import traceback
import os
import sys
import PyHelpers
import pickle
import CvGameUtils
import CvEventInterface
import CvWBPopups
import Popup as PyPopup
import OVPromotionUtils
import math


# for C++ compatibility
false=False
true=True

# globals
gc = CyGlobalContext()
localText = CyTranslator()
PyPlayer = PyHelpers.PyPlayer
PyInfo = PyHelpers.PyInfo
PyCity = PyHelpers.PyCity
PyGame = PyHelpers.PyGame
ArtFileMgr = CyArtFileMgr()

dArmyList = [
		"UNIT_ANCIENT_ARMY",
		"UNIT_CLASSICAL_ARMY",
		"UNIT_MEDIEVAL_ARMY",
		"UNIT_RENAISSANCE_ARMY",
		"UNIT_INDUSTRIAL_ARMY",
		"UNIT_MODERN_ARMY",
		"UNIT_FUTURE_ARMY"
	]

	
dArmyData = {
		"ERA_ANCIENT": (3,0,0,0,0,0,0),
		"ERA_CLASSICAL": (4,3,0,0,0,0,0),
		"ERA_MEDIEVAL": (5,4,3,0,0,0,0),
		"ERA_RENAISSANCE": (6,5,4,3,0,0,0),
		"ERA_INDUSTRIAL": (7,6,5,4,3,0,0),
		"ERA_MODERN": (8,7,6,5,4,3,0),
		"ERA_FUTURE": (9,8,7,6,5,4,3)
	}
	
dCombatList = [
		"UNITCOMBAT_MELEE",
		"UNITCOMBAT_ARCHER",
		"UNITCOMBAT_MOUNTED",
		"UNITCOMBAT_GUN",	
		"UNITCOMBAT_ARMOR"
	]
		
# ###################################
#	Begin Army Functions		#
# ###################################

def	getRequiredCitiesPerArmy():
	# Orion's Inquisition Mod
	return 3
			
def doMTIPlacementAI(pUnit):
	# Orion's Army Mod
	# Sends MTI unit to a city where the AI can build an Army
	iOwner = pUnit.getOwner()
	AIpPlayer = gc.getPlayer(iOwner)
	AICan = False
	
	#CyInterface().addImmediateMessage("A", "")
	for iCity in range(AIpPlayer.getNumCities()):
		pCity = AIpPlayer.getCity(iCity)	
		# Does AI city have enough units to build an Army? 
		if canAIBuildArmy(iOwner, pCity):
			AICity = pCity
			AIiCity = iCity
			AICan = True
			break
	
	if AICan:
		if pUnit.generatePath(AICity.plot(), 0, False, None):
			# Get the Era for the units in the AI city
			AISelectedEra = getAIArmyEra(iOwner, AIiCity)
						
			if pUnit.getX() != AICity.getX() or pUnit.getY() != AICity.getY():
				# Move the MTI to the AI city, where Army can be built.
				pUnit.getGroup().pushMission(MissionTypes.MISSION_MOVE_TO, AICity.getX(), AICity.getY(), 0, False, True, MissionAITypes.NO_MISSIONAI, pUnit.plot(), pUnit)
				
			else:
				doArmyUnitPreChecks(iOwner, AISelectedEra, pCity)
									
				
def doBuildArmy(pUnit):
	# Orion's Army Mod
	# Displays the Pick Era Popup for human player
	iOwner = pUnit.getOwner()
	pPlayer = gc.getPlayer(iOwner)
		
	if not pPlayer.isNone() and not pPlayer.isBarbarian() and pPlayer.isAlive():		
		if pPlayer.isHuman():
			# Let Human Player select the Era
			doPickEraPopup(iOwner)

	
def getTotalArmies(iPlayer):
	# Orion's Army Mod
	# Returns the total number of Army units
	iArmyUnitCount = 0
	pPlayer = gc.getPlayer(iPlayer)
	lUnits = PyPlayer( pPlayer.getID( ) ).getUnitList( )
	
	for iUnit in range(len(lUnits)):
		jUnitType = pPlayer.getUnit(lUnits[iUnit].getID()).getUnitType()
		if isTypeInList(jUnitType, dArmyList):
			iArmyUnitCount += 1
	
	return iArmyUnitCount
	

def cityCanTrainMTI(pCity):
	# Orion's Army Mod
	CanTrainMTI = False
	iOwner = pCity.getOwner()
	pPlayer = gc.getPlayer(iOwner)
	ArmyTech = gc.getInfoTypeForString("TECH_ORGANIZED_WARFARE")
	
	pTotalCities = pPlayer.getNumCities()
	TotalArmies = getTotalArmies(iOwner)
	iXx = pCity.getX()
	iYy = pCity.getY()
	pPlot = CyMap().plot(iXx, iYy)
	
	# Does the Player have the required Tech
	if gc.getTeam(pPlayer.getTeam()).isHasTech(ArmyTech):
		# Does the city have a Barracks?
		if pCity.isHasBuilding(gc.getInfoTypeForString("BUILDING_BARRACKS")):
			# Must not have any MTIs						
			if not hasMTIUnit(pPlot, iOwner):
				# Must have at least 3 cities to build first Army
				if pTotalCities > 2:		
					# Must have default number of cities for each army trained
					if  TotalArmies * getRequiredCitiesPerArmy() <= pTotalCities:
						#CyInterface().addImmediateMessage("Can Train Army", "")
						#if not isProducingMTI(iOwner, pCity):
						CanTrainMTI = True
							
	if CanTrainMTI:
		return True
	
	else:
		return False
	
	
def showBuildArmyButton(pUnit):
	# Orion's Army Mod
	iOwner = pUnit.getOwner()
	pUnitOwner = gc.getPlayer(iOwner)
	iUnitType = pUnit.getUnitType()
	MTI_Type = gc.getInfoTypeForString("UNIT_MTI")
	pTotalCities = pUnitOwner.getNumCities()
	TotalArmies = getTotalArmies(iOwner)
	zShowButton = False
		
	if iUnitType == MTI_Type:
		pCity = gc.getMap().plot(pUnit.getX(), pUnit.getY()).getPlotCity()
		iCityOwner = pCity.getOwner()
		pCityPlayer = gc.getPlayer(iCityOwner)
		
		if (iCityOwner == iOwner) or (gc.getTeam(pCityPlayer.getTeam()).isVassal(gc.getPlayer(iOwner).getTeam())):
			# Must have at least 3 cities to build first Army
			if pTotalCities > 2:
				# Must have default number of cities for each army trained
				if  TotalArmies * getRequiredCitiesPerArmy() <= pTotalCities:
					zShowButton = True

	return zShowButton	


def getMTIProductionChance(AIpPlayer):
	# Orion's Army Mod
	# Accelerates MTI Production for Aggressive AI traits
	iTrait = gc.getInfoTypeForString("TRAIT_AGGRESSIVE")
	
	if (AIpPlayer.hasTrait(iTrait)):
		# Aggressive AI Players get a 25% chance for Producing an MTI
		return 25
	else:
		# Non Aggressive AI Players get a 15% chance for Producing an MTI
		return 15
		

def isTypeInList(yType, dList):
	# Orion's Army Mod
	# Returns True if the yType is found in the dList
	# Returns False if the yType is not found in the dList
	bFoundType = False
	
	for szType in range(len(dList)):
		MyType = str(dList[szType])
		iType = gc.getInfoTypeForString(MyType)
				
		if yType == iType:
			bFoundType = True
			break
	
	if bFoundType:
		return True
	else:
		return False

				
def getUnitEra(pUnitXXType):
	# Orion's Army Mod
	# Returns the Era for a specified Unit type
	# Identify a valid Unit
	if pUnitXXType != -1:
		# Add exception for Warriors, who have no prerequisite Tech
		if pUnitXXType == gc.getInfoTypeForString("UNIT_WARRIOR"):
			# This makes Warriors appear on the selection list to create Army units
			XiEra = 0
			return XiEra
		else:
			# Get the Unit Info
			pUnitInfo = gc.getUnitInfo(pUnitXXType)
			
			if pUnitInfo != -1:
				# Get the prerequisit Tech for the Unit
				iTechType = pUnitInfo.getPrereqAndTech()
				
				if iTechType != -1:
					# Get the Tech Era
					XiEra = gc.getTechInfo(iTechType).getEra()
					
					if XiEra != -1:
						return XiEra
				

def hasUnitType(iPlayer, iUnitType):

	for pyUnit in PyHelpers.PyPlayer(iPlayer).getUnitList():
		if (pyUnit.getUnitType() == iUnitType):
			return True
			
	return False		
		
				
def hasMTIUnit(pPlot, iActivePlayer):
	# Orion's Army Mod
	# Returns True if the plot has an MTI unit 
	# Returns False if the plot does not have an MTI unit 
	FoundMTI = False	
		
	for iI in range (pPlot.getNumUnits()):
		# ID the Unit so we can check the type
		xUnit = pPlot.getUnit(iI)							
		ixUnitOwner = xUnit.getOwner()
		xUnitType = xUnit.getUnitType()
		
		if xUnitType == gc.getInfoTypeForString("UNIT_MTI"):
			if ixUnitOwner == iActivePlayer:
				#CyInterface().addImmediateMessage("Found MTI", "")
				FoundMTI = True
				break
		
	if FoundMTI:
		return True
	else:
		return False

	
def canBuildArmyCheck(iPlayer, iMaxUnits, iSelectedEra):
	# Orion's Army Mod
	# Returns True if the city has the minimum required units from the selected era to create an Army unit 
	# Returns False if the city does not have the minimum required units from the selected era to create an Army unit 
	pPlayer = gc.getPlayer(iPlayer)
	iUnitCount = 0
	
	for iPlotLoop in range(CyMap().numPlots()):
		pPlot = CyMap().plotByIndex(iPlotLoop)	
		pPlotiOwner = pPlot.getOwner()
		# Does the owner of the plot match the player?
		if pPlotiOwner == iPlayer:
			# Is the plot a city?
			if pPlot.isCity():
				# Does the city have an MTI?
				if hasMTIUnit(pPlot, pPlotiOwner):
					# Check each Unit on plot
					for iUnitLoop in range (pPlot.getNumUnits()):
						# Identify the Unit
						pUnitZ = pPlot.getUnit(iUnitLoop)
						pUnitZType = pUnitZ.getUnitType()
						pUnitZCombatType = pUnitZ.getUnitCombatType()
						# Exclude units not in Combat List
						if isTypeInList(pUnitZCombatType, dCombatList):
							# Get the Era for the Unit
							iEra = getUnitEra(pUnitZType)
							# Does the Unit Era match the Selected Era
							if iEra == iSelectedEra:
								if not pUnitZ.isHurt():
									iUnitCount = iUnitCount + 1
	
	if iUnitCount >= iMaxUnits:
		return True
		
	else:
		return False

		
def canAIBuildArmy(iPlayer, AICity):
	# Returns True if the AI city with enough units to build an Army.
	# Returns False if the AI city does not have enough units to build an Army.
	pPlayer = gc.getPlayer(iPlayer)
	iCurrentEra = pPlayer.getCurrentEra()
	FoundEligibleAICity = False
	iX = AICity.getX()
	iY = AICity.getY()
	AICityPlot = CyMap().plot(iX, iY)
				
	# Loop through Eras 
	for i in range(gc.getNumEraInfos()):
		if not i > iCurrentEra:
			iMaxUnits = getMaxUnits(iCurrentEra, i)					
			iUnitCount = 0
					
			for iUnitLoop in range (AICityPlot.getNumUnits()):
				# Identify the Unit
				pUnitZ = AICityPlot.getUnit(iUnitLoop)
				pUnitZType = pUnitZ.getUnitType()
				pUnitZCombatType = pUnitZ.getUnitCombatType()
				# Exclude units not in Combat List
				if isTypeInList(pUnitZCombatType, dCombatList):
					# Get the Era for the Unit
					iEra = getUnitEra(pUnitZType)
					# Does the Unit Era match the Selected Era
					if iEra == i:
						if not pUnitZ.isHurt():
							iUnitCount += 1
							if iUnitCount >= iMaxUnits:
								FoundEligibleAICity = True
								break
														
	
	if FoundEligibleAICity:
		return True
	
	return False
	
	
def getAIEligibleArmyCity(iPlayer):
	# Assumes AI can build an Army
	# Returns AICity if the city has enough units to build an Army.
	pPlayer = gc.getPlayer(iPlayer)
	iCurrentEra = pPlayer.getCurrentEra()
	FoundEligibleAICity = False
	
	if not pPlayer.isNone() and not pPlayer.isBarbarian() and not pPlayer.isHuman() and pPlayer.isAlive():
		# Loop through AI Cities 
		for iCity in range(pPlayer.getNumCities()):
			AICity = pPlayer.getCity(iCity)
			if FoundEligibleAICity == False:
				# Get City Plot
				iX = AICity.getX()
				iY = AICity.getY()
				AICityPlot = CyMap().plot(iX, iY)
							
				# Loop through Eras 
				for i in range(gc.getNumEraInfos()):
					if FoundEligibleAICity == False:
						if not i > iCurrentEra:
							iMaxUnits = getMaxUnits(iCurrentEra, i)
							iUnitCount = 0
							
							for iUnitLoop in range (AICityPlot.getNumUnits()):
								# Identify the Unit
								pUnitZ = AICityPlot.getUnit(iUnitLoop)
								pUnitZType = pUnitZ.getUnitType()
								pUnitZCombatType = pUnitZ.getUnitCombatType()
								# Exclude units not in Combat List
								if isTypeInList(pUnitZCombatType, dCombatList):
									# Get the Era for the Unit
									iEra = getUnitEra(pUnitZType)
									# Does the Unit Era match the Selected Era
									if iEra == i:
										if not pUnitZ.isHurt():
											iUnitCount += 1
											if iUnitCount >= iMaxUnits:
												FoundEligibleAICity = True
												return iCity
												break
												

def getAIArmyEra(iPlayer, AIiCity):	
	pPlayer = gc.getPlayer(iPlayer)
	iCurrentEra = pPlayer.getCurrentEra()
	FoundEligibleAICity = False
	
	if not pPlayer.isNone() and not pPlayer.isBarbarian() and not pPlayer.isHuman() and pPlayer.isAlive():
		# Loop through AI Cities 
		for izCity in range(pPlayer.getNumCities()):
			AICity = pPlayer.getCity(izCity)
			if izCity == AIiCity:
			
				# Get City Plot
				iX = AICity.getX()
				iY = AICity.getY()
				AICityPlot = CyMap().plot(iX, iY)
	
				# Loop through Eras 
				for i in range(gc.getNumEraInfos()):
					if not i > iCurrentEra:
						iMaxUnits = getMaxUnits(iCurrentEra, i)
						iUnitCount = 0
	
						for iUnitLoop in range (AICityPlot.getNumUnits()):
							# Identify the Unit
							pUnitZ = AICityPlot.getUnit(iUnitLoop)
							pUnitZType = pUnitZ.getUnitType()
							pUnitZCombatType = pUnitZ.getUnitCombatType()
							# Exclude units not in Combat List
							if isTypeInList(pUnitZCombatType, dCombatList):
								# Get the Era for the Unit
								iEra = getUnitEra(pUnitZType)
								# Does the Unit Era match the Selected Era
								if iEra == i:
									if not pUnitZ.isHurt():
										iUnitCount += 1
										if iUnitCount >= iMaxUnits:
											return iEra

		
def doArmyCheckFailurePopup(iPlayer):
	# Orion's Army Mod
	popupInfo = CyPopupInfo()
	popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON)
	szTitle = u"<font=4b>" + localText.getText("TXT_KEY_ARMY_POPUP_TITLE", ()) + u"</font>"
	szBody = u"<font=3>" + localText.getText("TXT_KEY_ARMY_CHECK_FAILURE", ()) + u"</font>"
	popupInfo.setText(szTitle + u"\n\n" + szBody)
	popupInfo.addPythonButton("OK", "")
	popupInfo.addPopup(iPlayer)
	
	
def doSiegeTowerAssaultMessagePopup(iPlayer):
	# Orion's Army Mod
	popupInfo = CyPopupInfo()
	popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON)
	szTitle = u"<font=4b>" + localText.getText("TXT_KEY_SIEGE_TOWER_INFO_POPUP_TITLE", ()) + u"</font>"
	szBody = u"<font=3>" + localText.getText("TXT_KEY_SIEGE_TOWER_MESSAGE", ()) + u"</font>"
	popupInfo.setText(szTitle + u"\n\n" + szBody)
	popupInfo.addPythonButton("OK", "")
	popupInfo.addPopup(iPlayer)

	
def doSiegeTowerAssaultFailurePopup(iPlayer):
	# Orion's Army Mod
	popupInfo = CyPopupInfo()
	popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON)
	szTitle = u"<font=4b>" + localText.getText("TXT_KEY_SIEGE_TOWER_INFO_POPUP_TITLE", ()) + u"</font>"
	szBody = u"<font=3>" + localText.getText("TXT_KEY_SIEGE_TOWER_FAILURE_MESSAGE", ()) + u"</font>"
	popupInfo.setText(szTitle + u"\n\n" + szBody)
	popupInfo.addPythonButton("OK", "")
	popupInfo.addPopup(iPlayer)
	

def doSiegeTowerAssaultSuccessPopup(iPlayer):
	# Orion's Army Mod
	popupInfo = CyPopupInfo()
	popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON)
	szTitle = u"<font=4b>" + localText.getText("TXT_KEY_SIEGE_TOWER_INFO_POPUP_TITLE", ()) + u"</font>"
	szBody = u"<font=3>" + localText.getText("TXT_KEY_SIEGE_TOWER_SUCCESS_MESSAGE", ()) + u"</font>"
	popupInfo.setText(szTitle + u"\n\n" + szBody)
	popupInfo.addPythonButton("OK", "")
	popupInfo.addPopup(iPlayer)	

	
def getNewCiv4ArmyUnit(iPlayer, iArmyUnitType, pPlot):
	# Orion's Army Mod
	pPlayer = gc.getPlayer(iPlayer)
	
	for iLoop in range (pPlot.getNumUnits()):
		TargetUnit = pPlot.getUnit(iLoop)
		TargetUnitUnitType = TargetUnit.getUnitType()
		
		if TargetUnitUnitType == iArmyUnitType:
			# Isolate the new Army from other Army units
			if TargetUnit.baseCombatStr() == 1:	
				break
				
	return TargetUnit

def createCiv4Army(iPlayer, iArmyUnitType):
	# Orion's Army Mod
	pPlayer = gc.getPlayer(iPlayer)
	iWestPoint = gc.getInfoTypeForString("BUILDING_WEST_POINT")
	iLeadership = gc.getInfoTypeForString("PROMOTION_LEADERSHIP")
	
	for iPlotLoop in range(CyMap().numPlots()):
		DefaultUnitAI = UnitAITypes.NO_UNITAI
		pPlot = CyMap().plotByIndex(iPlotLoop)		
		pPlotiOwner = pPlot.getOwner()
		# Does the owner of the plot match the player?
		if pPlotiOwner == iPlayer:
			# Is the plot a city?
			if pPlot.isCity():
				# Does the city have an MTI?
				if hasMTIUnit(pPlot, pPlotiOwner):
					pCity = pPlot.getPlotCity()
					iX = pCity.getX()
					iY = pCity.getY()
					pPlayer.initUnit(iArmyUnitType, iX, iY, DefaultUnitAI, DirectionTypes.NO_DIRECTION)
					#CyInterface().addImmediateMessage("Initiate Army", "")
					# Does the city have the West Point National Wonder?
					if pCity.isHasBuilding(iWestPoint):
						# Get the new Army unit
						MyNewArmyUnit = getNewCiv4ArmyUnit(iPlayer, iArmyUnitType, pPlot)
						# Add Leadership promotion to Armies created in city with West Point National Wonder
						MyNewArmyUnit.setHasPromotion(iLeadership, True)
						break
					
					else:
						break
						
def createCiv4AIArmy(iPlayer, iArmyUnitType, AIpCity):			
	# Orion's Army Mod
	pPlayer = gc.getPlayer(iPlayer)
	iWestPoint = gc.getInfoTypeForString("BUILDING_WEST_POINT")
	iLeadership = gc.getInfoTypeForString("PROMOTION_LEADERSHIP")
	DefaultUnitAI = UnitAITypes.NO_UNITAI
	iX = AIpCity.getX()
	iY = AIpCity.getY()
	AICityPlot = CyMap().plot(iX, iY)
	
	pPlayer.initUnit(iArmyUnitType, iX, iY, DefaultUnitAI, DirectionTypes.NO_DIRECTION)
	#CyInterface().addImmediateMessage("Initiate Army", "")
	# Does the city have the West Point National Wonder?
	if AIpCity.isHasBuilding(iWestPoint):
		# Get the new Army unit
		MyNewArmyUnit = getNewCiv4ArmyUnit(iPlayer, iArmyUnitType, AICityPlot)
		# Add Leadership promotion to Armies created in city with West Point National Wonder
		MyNewArmyUnit.setHasPromotion(iLeadership, True)
								
						
def doGetAIEraUnits(iPlayer, iSelectedEra, AIpCity):
	# Orion's Army Mod		
	pPlayer = gc.getPlayer(iPlayer)
	ProcessAIArmyUnit = False
	iX = AIpCity.getX()
	iY = AIpCity.getY()
	pCityPlot = CyMap().plot(iX, iY)
	pArmyMoves = 2
	iCombinedUnitCount = 0	
	
	global pArmyCombat
	global iArmyUnitType
	global iTotalPromotions
	global iTotalUnitsDivider	
	
	# Identify the virgin Army Unit
	MyNewArmyUnit = getNewCiv4ArmyUnit(iPlayer, iArmyUnitType, pCityPlot)	
	# Check each Unit on the city plot	
	
	for icount in range(iTotalUnitsDivider):
		# Counter
		iCombinedUnitCount = iCombinedUnitCount + 1
		#CyInterface().addImmediateMessage(str(iCombinedUnitCount), "")
		
		for iUnitLoop in range (pCityPlot.getNumUnits()):
			# Identify the Unit
			pUnitZ = pCityPlot.getUnit(iUnitLoop)
			pUnitZType = pUnitZ.getUnitType()
			pUnitZCombatType = pUnitZ.getUnitCombatType()
			# Exclude units not in Combat List
			if isTypeInList(pUnitZCombatType, dCombatList):
				# Get the Era for the Unit
				iEra = getUnitEra(pUnitZType)
				# Does the Unit Era match the Selected Era
				if iEra == iSelectedEra:
					# Make sure only undamaged units can be selected
					if not pUnitZ.isHurt():				
						# 1. Record Combat Strength
						pArmyCombat += pUnitZ.baseCombatStr()					
						# 2. Record the total number of Promotions for the selected unit
						iTotalPromotions += OVPromotionUtils.getTotalUnitPromotions(pUnitZ)				
						# 3. Transfer all promotions from unit to Army Unit
						doTransferPromotions(pUnitZ, MyNewArmyUnit)			
						# 4. Record moves to 1 if selected unit moves only 1
						if pUnitZ.baseMoves() == 1:
							pArmyMoves = 1					
								
						# Delete Unit
						pUnitZ.kill( 0, -1 )			
						#CyInterface().addImmediateMessage("A", "")
						ProcessAIArmyUnit = True
						break
								

	if ProcessAIArmyUnit:
		# Calculate the promotion rate
		iSinglePromotionRate = OVPromotionUtils.getPromotionRate(iTotalUnitsDivider, iTotalPromotions)
		
		for iLoop in range (pCityPlot.getNumUnits()):
			TargetUnit = pCityPlot.getUnit(iLoop)
			TargetUnitType = TargetUnit.getUnitType()
			
			if TargetUnitType == iArmyUnitType:				
				if TargetUnit.baseCombatStr() == 1:					
					RealStrength = pArmyCombat - 1
					#CyInterface().addImmediateMessage(RealStrength, "")					
					MyNewArmyUnit.setBaseCombatStr(RealStrength)
					MyNewArmyUnit.changeMoves(pArmyMoves)
					# Run Promotion repeals
					doPromotionRepeals(AIpCity, MyNewArmyUnit, iSinglePromotionRate)
					
			elif TargetUnitType == gc.getInfoTypeForString("UNIT_MTI"):
				# Unit expended -- Delete MTI Unit
				TargetUnit.kill(0, -1)
	
	
	
def doPickUnitsPopup(iPlayer, iSelectedEra):
	# Orion's Army Mod	
	#CyInterface().addImmediateMessage(str(iMaxUnits), "")
	pPlayer = gc.getPlayer(iPlayer)
		
	popupInfo = CyPopupInfo()
	popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON)
	szTitle = u"<font=4b>" + localText.getText("TXT_KEY_ARMY_POPUP_TITLE", ()) + u"</font>"
	szBody = u"<font=3>" + localText.getText("TXT_KEY_SELECT_ARMY_UNITS", ()) + u"</font>"
	popupInfo.setText(szTitle + u"\n\n" + szBody)	
	#popupInfo.setText(CyTranslator().getText("TXT_KEY_SELECT_ARMY_UNITS",()))
	popupInfo.setData1(iPlayer)
	popupInfo.setOnClickedPythonCallback("selectUnitCallback")
	
	#Populate the Popup
	for iPlotLoop in range(CyMap().numPlots()):
		pPlot = CyMap().plotByIndex(iPlotLoop)		
		#pUnit = pPlot.getUnit(iUnitLoop)
		pPlotiOwner = pPlot.getOwner()
		# Does the owner of the plot match the player?
		if pPlotiOwner == iPlayer:
			# Is the plot a city?
			if pPlot.isCity():
				# Does the city have an MTI?
				if hasMTIUnit(pPlot, pPlotiOwner):
					#CyInterface().addImmediateMessage(str(apUnitList), "")
					# Check each Unit on plot
					for iUnitLoop in range (pPlot.getNumUnits()):
						# Identify the Unit
						pUnitZ = pPlot.getUnit(iUnitLoop)
						pUnitZType = pUnitZ.getUnitType()
						pUnitZCombatType = pUnitZ.getUnitCombatType()
						# Exclude units not in Combat List
						if isTypeInList(pUnitZCombatType, dCombatList):
							# Get the Era for the Unit
							iEra = getUnitEra(pUnitZType)
							# Does the Unit Era match the Selected Era 
							if iEra == iSelectedEra:
								# Make sure only undamaged units can be selected
								if not pUnitZ.isHurt():
									# Store the era for later use
									popupInfo.setData2(iEra)
									# Display filtered list of units in popup
									popupInfo.addPythonButton(gc.getUnitInfo(pUnitZType).getDescription(), gc.getUnitInfo(pUnitZType).getButton())
									#CyInterface().addImmediateMessage("Here", "")
											
	popupInfo.addPopup(iPlayer)
	
		
def doPickEraPopup(iPlayer):
	# Orion's Army Mod
	# For Human Players
	iCount = 0
	pPlayer = gc.getPlayer(iPlayer)
	iCurrentEra = pPlayer.getCurrentEra()
	
	popupInfo = CyPopupInfo()
	popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON)
	szTitle = u"<font=4b>" + localText.getText("TXT_KEY_ARMY_POPUP_TITLE", ()) + u"</font>"
	szBody = u"<font=3>" + localText.getText("TXT_KEY_SELECT_ARMY_ERA", ()) + u"</font>"
	popupInfo.setText(szTitle + u"\n\n" + szBody)
	#popupInfo.setText(CyTranslator().getText("TXT_KEY_SELECT_ARMY_ERA",()))
	popupInfo.setData1(iPlayer)
	popupInfo.setOnClickedPythonCallback("selectArmyEraCallback")	
	
	for i in range(gc.getNumEraInfos()):
		# Reads "Ancient"
		EraDescription = gc.getEraInfo(i).getDescription()
		# Reads "ERA_ANCIENT"
		EraType = gc.getEraInfo(i).getType()		
		strButtonText = str(EraDescription)
		MyArt = "INTERFACE_" + str(EraType)
		popupInfo.setData2(i)
		
		if not i > iCurrentEra:
			iCount = iCount + 1			
			popupInfo.addPythonButton(strButtonText, ArtFileMgr.getInterfaceArtInfo(MyArt).getPath())		
			#CyInterface().addImmediateMessage(str(i), "")
			
	popupInfo.setData3(iCount)
	popupInfo.addPythonButton("Cancel", "")
	popupInfo.addPopup(iPlayer)


def addPopup(szTitle, szText):
	# Orion's Army Mod 
	# Don't display popups for autoplay games
	if (gc.getPlayer(CyGame().getActivePlayer()).isAlive()):
	
		popup = PyPopup.PyPopup(-1)
		popup.setHeaderString(szTitle)
		popup.setBodyString(szText)
		popup.launch(true, PopupStates.POPUPSTATE_QUEUED)


def doArmyUnitPreChecks(iPlayer, iSelectedEra, pCity):
	# Orion's Army Mod
	# ********** For AI Players ************
	pPlayer = gc.getPlayer(iPlayer)
	iCurrentEra = pPlayer.getCurrentEra()
	MyMaxUnits = getMaxUnits(iCurrentEra, iSelectedEra)
	# Get ArmyTpye based on selected Era
	iArmyType = getArmyType(iSelectedEra)
	AIpTotalCities = pPlayer.getNumCities()
	AITotalArmies = getTotalArmies(iPlayer)
		
	# Run Check
	if canAIBuildArmy(iPlayer, pCity):
		# Must have at least 3 cities to build first Army
		if AIpTotalCities > 2:
			# Must have default number of cities for each army trained
			if AITotalArmies * getRequiredCitiesPerArmy() <= AIpTotalCities:
	
				#CyInterface().addImmediateMessage("A", "")			
				global pUnitsToDelete
				global pArmyCombat
				global iArmyUnitType
				global iTotalPromotions
				global iTotalUnitsDivider
				
				pUnitsToDelete = MyMaxUnits
				pArmyCombat = 1
				iArmyUnitType = iArmyType
				iTotalPromotions = 0
				iTotalUnitsDivider = MyMaxUnits
				# Create Army Unit	
				createCiv4AIArmy(iPlayer, iArmyUnitType, pCity)
				doGetAIEraUnits(iPlayer, iSelectedEra, pCity)
		
	
def getMaxUnits(iCurrentEra, iSelectedEra):
	# Orion's Army Mod 
	# Returns the Maximum number of units that can be added into the Army
			
	for szEra, (iAncient, iClassical, iMedieval, iRenaissance, iIndustrial, iModern, iFuture) in dArmyData.iteritems():
		iTechEra = gc.getInfoTypeForString(szEra)
		# Select the data line that matches the player's current ERA
		if iTechEra == iCurrentEra:
			break
		
	# 0 = ERA_ANCIENT
	if iSelectedEra == 0:		
		return iAncient
		
	# 1 = ERA_CLASSICAL		
	if iSelectedEra == 1:
		return iClassical
		
	# 2 = ERA_MEDIEVAL	
	if iSelectedEra == 2:
		return iMedieval
		
	# 3 = ERA_RENAISSANCE	
	if iSelectedEra == 3:
		return iRenaissance
		
	# 4 = ERA_INDUSTRIAL	
	if iSelectedEra == 4:
		return iIndustrial
		
	# 5 = ERA_MODERN	
	if iSelectedEra == 5:
		return iModern
		
	# 6 = ERA_FUTURE	
	if iSelectedEra == 6:
		return iFuture

		
def getArmyType(iEra):
	# Orion's Army Mod	
	MyArmyType = gc.getInfoTypeForString("UNIT_ANCIENT_ARMY")
	
	# 0 = ERA_ANCIENT
	if iEra == 0:
		MyArmyType = gc.getInfoTypeForString("UNIT_ANCIENT_ARMY")
			
	# 1 = ERA_CLASSICAL
	if iEra == 1:
		MyArmyType = gc.getInfoTypeForString("UNIT_CLASSICAL_ARMY")
			
	# 2 = ERA_MEDIEVAL
	if iEra == 2:
		MyArmyType = gc.getInfoTypeForString("UNIT_MEDIEVAL_ARMY")
			
	# 3 = ERA_RENAISSANCE
	if iEra == 3:
		MyArmyType = gc.getInfoTypeForString("UNIT_RENAISSANCE_ARMY")
			
	# 4 = ERA_INDUSTRIAL
	if iEra == 4:
		MyArmyType = gc.getInfoTypeForString("UNIT_INDUSTRIAL_ARMY")
			
	# 5 = ERA_MODERN
	if iEra == 5:
		MyArmyType = gc.getInfoTypeForString("UNIT_MODERN_ARMY")		
	
	# 6 = ERA_FUTURE
	if iEra == 6:
		MyArmyType = gc.getInfoTypeForString("UNIT_FUTURE_ARMY")
	
	return MyArmyType
	
	
def doTransferPromotions(FromUnit, ToArmyUnit):	
	# Orion's Army Mod 
	iGreatGeneral = gc.getInfoTypeForString("UNIT_GREAT_GENERAL")

	# Loop through promotions
	for iProm in range(gc.getNumPromotionInfos()):
		# Does the unit have this promotion
		if FromUnit.isHasPromotion(iProm):				
			# Army unit must not already have this promotion
			if not ToArmyUnit.isHasPromotion(iProm):
				# Transfer promotion to the New Army Unit
				ToArmyUnit.setHasPromotion(iProm, True)					
				# Get the Leader Type
				FromUnitLeaderType = FromUnit.getLeaderUnitType()
				# Does the unit have a leader?
				if FromUnitLeaderType != -1:
					# Get the Leader Name
					FromUnitName = FromUnit.getNameNoDesc()
					#CyInterface().addImmediateMessage(str(FromUnitName), "")
					# Get the Leader Type for Army Unit
					MyNewArmyUnitLeaderType = ToArmyUnit.getLeaderUnitType()
					# Make sure the Army unit does not already have a leader
					if MyNewArmyUnitLeaderType == -1:
						ToArmyUnit.setLeaderUnitType(iGreatGeneral)
						ToArmyUnit.setName(FromUnitName)

						
def doPromotionRepeals(pCity, MyNewArmyUnit, iSinglePromotionRate):
	# Orion's Army Mod 
	# Removes promotions from new Army unit
	# Promotion removal is based upon randum chance 
	iSentry = gc.getInfoTypeForString("PROMOTION_SENTRY")
	iWestPoint = gc.getInfoTypeForString("BUILDING_WEST_POINT")
	iLeadership = gc.getInfoTypeForString("PROMOTION_LEADERSHIP")
	OVPromotionUtils.runGroupListCheck(MyNewArmyUnit)
						
	for iProm in range(gc.getNumPromotionInfos()):
		if MyNewArmyUnit.isHasPromotion(iProm):
			PromotionGroup = OVPromotionUtils.getPromotionGroup(iProm)
			# Select promotions that are not in a promotion group 
			if str(PromotionGroup) == "NONE":
				# Exclude Sentry Promotion, which all Army units get for free.
				if iProm != iSentry:										
					#CyInterface().addImmediateMessage("X", "")
					# Run calculated chance of loosing the promotion 										
					if not OVPromotionUtils.doRandumChance(iSinglePromotionRate):
						# Does the city have the West Point National Wonder?
						if pCity.isHasBuilding(iWestPoint):
							# Exclude the leadership promotion
							if iProm != iLeadership:
								# Remove Promotion
								MyNewArmyUnit.setHasPromotion(iProm, false)
								
						else:
							# Remove Promotion
							MyNewArmyUnit.setHasPromotion(iProm, false)

# Siege Tower Functions

def isAdjacentCityPlot(pUnit):
	# Orion's Army Mod
	# Returns True if the unit is on a plot adjacent to a city
	# Returns False if the unit is Not on a plot adjacent to a city
	isAdjacentPlot = False
	iOwner = pUnit.getOwner()
	pPlayer = gc.getPlayer(iOwner)
	pTeam = gc.getTeam(iOwner)
	iPlotX = pUnit.getX()
	iPlotY = pUnit.getY()
	pUnitPlot = CyMap().plot(iPlotX, iPlotY)	
	
	for iDirection in range(DirectionTypes.NUM_DIRECTION_TYPES):
		pLoopPlot = plotDirection( iPlotX, iPlotY, DirectionTypes(iDirection))
		iLoopPlotOwner = pLoopPlot.getOwner()
		LoopPlotOwner = gc.getPlayer(iLoopPlotOwner)
						
		# make sure it is not None
		if not pLoopPlot.isNone():
			if iLoopPlotOwner != iOwner:
				if pLoopPlot != pUnitPlot:
					if pLoopPlot.isCity():
						# Must be at war to attack
						if pTeam.isAtWar(LoopPlotOwner.getTeam()):
												
							#CyInterface().addImmediateMessage("Found Adjacent Plot", "")
							isAdjacentPlot = True
							break

	if isAdjacentPlot:
		return True
	
	return False
	
											
def showTowerAssaultButton(pUnit):
	# Orion's Siege Tower Mod	
	iOwner = pUnit.getOwner()
	pUnitOwner = gc.getPlayer(iOwner)
	iUnitType = pUnit.getUnitType()
	iSiegeTowerType = gc.getInfoTypeForString("UNIT_SIEGE_TOWER")
	zShowButton = False
	
	if iUnitType == iSiegeTowerType:		
		if isAdjacentCityPlot(pUnit):		
			#CyInterface().addImmediateMessage("A", "")				
			zShowButton = True

	return zShowButton
	

def doTowerAssault(pUnit):
	# Orion's Army Mod
	iOwner = pUnit.getOwner()
	pPlayer = gc.getPlayer(iOwner)
	pTeam = gc.getTeam(iOwner)
	iPlotX = pUnit.getX()
	iPlotY = pUnit.getY()
	pUnitPlot = CyMap().plot(iPlotX, iPlotY)
	iWalls = gc.getInfoTypeForString("BUILDING_WALLS")
	iCastle = gc.getInfoTypeForString("BUILDING_CASTLE")
		
	if not pPlayer.isNone() and not pPlayer.isBarbarian() and pPlayer.isAlive():	
		for iDirection in range(DirectionTypes.NUM_DIRECTION_TYPES):
			pLoopPlot = plotDirection( iPlotX, iPlotY, DirectionTypes(iDirection))
			iLoopPlotOwner = pLoopPlot.getOwner()
			LoopPlotOwner = gc.getPlayer(iLoopPlotOwner)
											
			# make sure it is not None
			if not pLoopPlot.isNone():
				if iLoopPlotOwner != iOwner:
					if pLoopPlot != pUnitPlot:
						if pLoopPlot.isCity():
							if pTeam.isAtWar(LoopPlotOwner.getTeam()):
								AssaultCity = pLoopPlot.getPlotCity()
								if AssaultCity.isHasBuilding(iCastle):
									# 33 percent chance to remove a Castle and Walls
									if OVPromotionUtils.doRandumChance(33):
										# Removes Caste and Walls
										AssaultCity.setNumRealBuilding (iCastle, 0)
										AssaultCity.setNumRealBuilding (iWalls, 0)
										# Get Randum Chance
										DamageChance = CyGame().getSorenRandNum(100,"")
										
										# Set Damage to the Unit
										if DamageChance > 0 and DamageChance < 33:
											pUnit.setDamage(50, iOwner)
										
										if DamageChance > 33 and DamageChance < 66:
											pUnit.setDamage(66, iOwner)
										
										if DamageChance > 66 and DamageChance < 100:
											pUnit.setDamage(75, iOwner)
										
										# Is this a Human player?
										if pPlayer.isHuman():
											doSiegeTowerAssaultSuccessPopup(iOwner)
									
									else:
										# Siege Attempt failed									
										# Is this a Human player?
										if pPlayer.isHuman():
											doSiegeTowerAssaultFailurePopup(iOwner)
											# Unit expended
											pUnit.kill(0, -1)
										
										else:
											# Unit expended
											pUnit.kill(0, -1)
								
								elif AssaultCity.isHasBuilding(iWalls):
									# 50 percent chance to remove the city Walls
									if OVPromotionUtils.doRandumChance(50):
										AssaultCity.setNumRealBuilding (iWalls, 0)										
										DamageChance = CyGame().getSorenRandNum(100,"")
										
										# Set Damage to the Unit
										if DamageChance > 0 and DamageChance < 33:
											pUnit.setDamage(50, iOwner)
										
										if DamageChance > 33 and DamageChance < 66:
											pUnit.setDamage(66, iOwner)
										
										if DamageChance > 66 and DamageChance < 100:
											pUnit.setDamage(75, iOwner)									
										
										# Is this a Human player?
										if pPlayer.isHuman():
											doSiegeTowerAssaultSuccessPopup(iOwner)
										
									else:
										# Siege Attempt failed
										# Is this a Human player?
										if pPlayer.isHuman():
											doSiegeTowerAssaultFailurePopup(iOwner)
											# Unit expended
											pUnit.kill(0, -1)
											
										else:
											# Unit expended
											pUnit.kill(0, -1)
								
								else:
									if pPlayer.isHuman():
										doSiegeTowerAssaultMessagePopup(iOwner)
										#CyInterface().addImmediateMessage("B", "")
		
# End Orion's Army Mod Functions	

