# Sid Meier's Civilization 4
# Copyright Firaxis Games 2005
# Orion's Army Mod: Promotion Utilities
from CvPythonExtensions import *
import BugUtil
import BugCore
import CvUtil
import CvTechChooser
import traceback
import os
import sys
import PyHelpers
import pickle
import CvGameUtils
import CvEventInterface
import CvWBPopups
import Popup as PyPopup


# for C++ compatibility
false=False
true=True

# globals
gc = CyGlobalContext()
localText = CyTranslator()
PyPlayer = PyHelpers.PyPlayer
PyInfo = PyHelpers.PyInfo
PyCity = PyHelpers.PyCity
PyGame = PyHelpers.PyGame


# ###################################
#	Begin Promotion Functions		#
# ###################################

def doRandumChance(Number):
	# Orion's Army Mod
	# Produces a randum number between 0 and 100
	chance = CyGame().getSorenRandNum(100,"")
		
	if chance <= Number:
		return True
	else:
		return False
		
		
def isGroupPromotion(ipzPromotion):
	# Orion's Army Mod
	# Returns True if a Promotion is a Group Promotion
	# Returns False if a Promotion is not a Group Promotion
	#Promotion = gc.getPromotionInfo(ipzPromotion).getDescription()
	Promotion = gc.getPromotionInfo(ipzPromotion).getType()
	if Promotion[-1] in map(str,range(10)):
		# Promotion is a Group Promotion
		#CyInterface().addImmediateMessage(str(Promotion), "")
		return True
				
	return False

		
def getPromotionRate(iMaxUnits, iTotalPromotions):
	# Orion's Army Mod
	# Returns a calculated Promotion rate
	if iTotalPromotions == 0:
		MyRate = 0
		
	else:
		MyRate = abs(iTotalPromotions / iMaxUnits * .33333333333 * 100)

		if MyRate > 0:
			if MyRate > 100:
				MyRate = 100
				
	return str(MyRate)
	
	
def getTotalUnitPromotions(pUnit):
	# Orion's Army Mod
	' Total count of Unit Promotions '
	iPromTotal = 0
	
	for iPromotion in range(gc.getNumPromotionInfos()):
		# Does the unit have this promotion
		if pUnit.isHasPromotion(iPromotion):	
			iPromTotal += 1
	
	return iPromTotal

	
def getPromotionLevel(iProm):
	# Orion's Army Mod
	# Returns the Promotion Level for the specified Promotion
	if isGroupPromotion(iProm):	
		Promotion = gc.getPromotionInfo(iProm).getType()
		ProLevel = Promotion[-1]
		return str(ProLevel)
		
	else:
		return str("1")	
	
	
def getPromotionGroup(iProm):
	# Orion's Army Mod
	# Strips off "PROMOTION_" and the Promotion level from the specified group Promotion
	# Returns the Promotion Group
	# Returns "NONE" for single promotions
	if isGroupPromotion(iProm):
		Promotion = gc.getPromotionInfo(iProm).getType()
		StripPromotion = Promotion.strip("PROMOTION_")
		ProGrp = StripPromotion.rstrip(str(StripPromotion[-1]))
		return str(ProGrp)
		
	else:	
		return "NONE"	
	
	
def getHighestPromotionLevel(pGroup):
	# Orion's Army Mod
	# Returns the Highest Promotion Level for the specified promotion group
	iLevelCount = 0

	for iProm in range(gc.getNumPromotionInfos()):
		MyProGroup = getPromotionGroup(iProm)
		if MyProGroup == pGroup:
			iLevelCount += 1
	
	return iLevelCount
						
						
def runGroupListCheck(kUnit):						
	# Orion's Army Mod
	lGroups = list()
	
	for iProm in range(gc.getNumPromotionInfos()):
		strGroup = getPromotionGroup(iProm)
		# Setup a summary list of group promotions  
		if strGroup not in lGroups:
			lGroups.append(strGroup)
			# Does the Unit have this group promotion? 
			for iPr in range(gc.getNumPromotionInfos()):
				if kUnit.isHasPromotion(iPr):
					GroupName = getPromotionGroup(iPr)
					#CyInterface().addImmediateMessage(str(GroupName), "")
					if GroupName == strGroup:
						# Yup!  Run promotion group checks.
						doPromotionGroupRandumChecks(kUnit, strGroup)
						#CyInterface().addImmediateMessage("Here I am!", "")
						break


def doPromotionGroupRandumChecks(kUnit, kGroup):
	# Orion's Army Mod
	iMax = getHighestPromotionLevel(kGroup)
	ContinueSearch = True
	#CyInterface().addImmediateMessage(str(iMax), "")
		
	for k in range(iMax, 0, -1):
		if ContinueSearch:
			for iPromotion in range(gc.getNumPromotionInfos()):
				#strGroup = getPromotionGroup(iPrm)				
				if kUnit.isHasPromotion(iPromotion):
					iPromotionLevel = getPromotionLevel(iPromotion)
					if iPromotionLevel == k:
						iSinglePromotionRate = getPromotionRate(iTotalUnitsDivider, iTotalPromotions)
						
						if k == 6:
							# promotion level = Single Rate
							GroupChance = iSinglePromotionRate
							if not doRandumChance(iSinglePromotionRate):
								kUnit.setHasPromotion(iPromotion, false)
								break
								
						elif k == 5:
							# promotion level = Single Rate * 1.2
							GroupChance = iSinglePromotionRate * 1.2
							if not doRandumChance(GroupChance):
								kUnit.setHasPromotion(iPromotion, false)
								break							
								
						elif k == 4:
							# promotion level = Single Rate * 1.4
							GroupChance = iSinglePromotionRate * 1.4
							if not doRandumChance(GroupChance):
								kUnit.setHasPromotion(iPromotion, false)
								break
						# For promotions at level 3		
						elif k == 3:
							# promotion level = Single Rate * 1.6
							GroupChance = iSinglePromotionRate * 1.6
							if not doRandumChance(GroupChance):
								kUnit.setHasPromotion(iPromotion, false)
								break
						# For promotions at level 2			
						elif k == 2:
							# promotion level = Single Rate * 1.8
							GroupChance = iSinglePromotionRate * 1.8
							if not doRandumChance(GroupChance):
								kUnit.setHasPromotion(iPromotion, false)
								break
						# For promotions at level 1			
						elif k == 1:
							iPrereq = gc.getPromotionInfo(iPromotion).getPrereqOrPromotion1()
							# promotion level = Single Rate * 2
							GroupChance = iSinglePromotionRate * 2
							
							if iPrereq != -1:
								# Make sure the prerequisite promotion was not deleted
								if not kUnit.isHasPromotion(iPrereq):
									kUnit.setHasPromotion(iPrereq, True)							
							
							if not doRandumChance(GroupChance):
								kUnit.setHasPromotion(iPromotion, False)												
								break
								
						else:
							ContinueSearch = False
							break
							
		else:
			break

# End Orion's Army Mod Promotion Utilities	

